/* -*- linux-c -*- */
/*
 * 緊急地震速報音検出プログラム
 *
 * copyright (c) 2013-2017 Kunihiko IMAI <bak@d2.dion.ne.jp>
 * ALL RIGHTS RESERVED
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>


#define NMEMB(ary) (sizeof(ary)/sizeof(ary[0]))
/* 4k */
unsigned chime_table[] = {
#include "chimetable.dat"
};


unsigned tap[NMEMB(chime_table)], corr[NMEMB(chime_table)];

inline void shift_tap ( int bit )
{
	int i;

	for ( i = 0; i < NMEMB(tap)-1; ++i ) {
		tap[i] <<= 1;
		tap[i] |= ( tap[i+1] & 0x80000000 ) ? 1 : 0;
	}
	tap[i] <<= 1;
	tap[i] |= ( bit ) ? 1 : 0;
}


inline int do_corr ( void )
{
	int i;

	for ( i = 0; i < NMEMB(tap); ++i ) {
		corr[i] = chime_table[i] ^ tap[i];
	}
}

inline unsigned count_bits ( unsigned bits )
{
	bits = (bits & 0x55555555) + (bits >> 1 & 0x55555555);
	bits = (bits & 0x33333333) + (bits >> 2 & 0x33333333);
	bits = (bits & 0x0f0f0f0f) + (bits >> 4 & 0x0f0f0f0f);
	bits = (bits & 0x00ff00ff) + (bits >> 8 & 0x00ff00ff);
	return (bits & 0x0000ffff) + (bits >>16 & 0x0000ffff);
}


inline int count_bits_array ( unsigned ary[NMEMB(chime_table)] )
{
	int i, r = 0;

	for ( i = 0; i < NMEMB(chime_table); ++i ) {
		r += count_bits(ary[i]);
	}
	return r;
}

inline void init_tap ( void )
{
	memset ( tap, 0, sizeof(tap) );
	memset ( corr, 0, sizeof(corr));
}


int main ( int argc, char *argv[] )
{
	int a0, a1, a, c;
	int t_mask = 0;
	int debug_out = 0;
	int threshold = 2200;
	int i = 1;

	while ( i < argc ) {
		if ( strcmp("-d", argv[i]) == 0 ) {
			debug_out = 1;
			++i;
		} else if ( strcmp ( "-t", argv[i] ) == 0 ) {
			++i;
			if ( i >= argc ) {
				fprintf ( stderr, "-t needs threshold value.\n" );
				return -1;
			}
			threshold = atoi ( argv[i] );
			++i;
		} else {
			fprintf ( stderr, "unknown option.\n" );
			return -1;
		}
	}



	init_tap ();
	for(;;) {
		a0 = getchar();
		if ( a0 < 0 )
			break;
		a1 = getchar();
		if ( a1 < 0 )
			break;
		a = a0 + a1;

		shift_tap ( a & 0x100 );
		do_corr ();
		c = count_bits_array( corr );
		if ( c < threshold && t_mask <= 0 ) {
			printf ( "EEW is detected.\n" );
			fflush ( stdout );
			t_mask = 60*4000;
		}
		if ( debug_out )
			fprintf ( stderr, "%d\n", c );
		if ( t_mask > 0 )
			--t_mask;
	}
}
